package rition.backend.api.v1.dto.response;

import com.fasterxml.jackson.annotation.JsonInclude;
import lombok.Data;
import rition.common.exception.code.ServiceCode;

import java.util.Objects;

/**
 * 通用的响应
 */
@Data
@JsonInclude(JsonInclude.Include.NON_NULL)
public class Response<T> {

    /**
     * 业务响应码，用于表示请求处理的结果
     */
    protected final int code;

    /**
     * 响应信息文本
     */
    protected final String message;

    /**
     * 响应数据体
     */
    protected final T data;

    /**
     * @param code    业务响应码，用于表示请求处理的结果
     * @param message 响应信息文本
     * @param data    响应数据体
     */
    public Response(int code, String message, T data) {
        this.code = code;
        this.message = message;
        this.data = data;
    }

    public Response(Response<T> response) {
        this.code = response.code;
        this.message = response.message;
        this.data = response.data;
    }

    public static <T> Response<T> success() {
        return success(null);
    }

    public static <T> Response<T> success(T data) {
        return new Response<>(ServiceCode.Ok.getCode(), "ok", data);
    }

    public static <T> Response<T> success(int code, T data) {
        return new Response<>(code, "ok", data);
    }

    public static <T> Response<T> success(int code, String msg, T data) {
        return new Response<>(code, msg, data);
    }

    public static <T> Response<T> error(ServiceCode code) {
        return new Response<>(code.getCode(), code.getMsg(), null);
    }

    public static <T> Response<T> error(int code) {
        var codes = ServiceCode.values();
        for (var serviceCode : codes) {
            if (serviceCode.getCode() == code) {
                return Response.error(serviceCode);
            }
        }

        return Response.error(ServiceCode.UnknownErr);
    }

    public static <T> Response<T> error(int code, String message) {
        return new Response<>(code, message, null);
    }

    public static <T> Response<T> error(ServiceCode code, String message) {
        return new Response<>(code.getCode(), message, null);
    }

    public static <T> Response<T> error(int code, String message, T data) {
        return new Response<>(code, message, data);
    }

    public static <T> Response<T> error(ServiceCode code, String message, T data) {
        return new Response<>(code.getCode(), message, data);
    }

    public int code() {
        return code;
    }

    public String message() {
        return message;
    }

    public T data() {
        return data;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) return true;
        if (obj == null || obj.getClass() != this.getClass()) return false;
        var that = (Response<?>) obj;
        return this.code == that.code &&
                Objects.equals(this.message, that.message) &&
                Objects.equals(this.data, that.data);
    }

    @Override
    public int hashCode() {
        return Objects.hash(code, message, data);
    }

    @Override
    public String toString() {
        return "Response[" +
                "code=" + code + ", " +
                "Message=" + message + ", " +
                "data=" + data + ']';
    }

}